@@ -26,6 +26,10 @@ from promnesia.sources import telegram
 from promnesia.sources import takeout, instapaper, pocket, fbmessenger, twitter, roamresearch, hypothesis, rss
 
 
+# NOTE: at the moment try to avoid using complex sources names
+# it's best to stick to digits, latin characters, dashes and underscores
+# there might be some UI problems related to that
+
 # now we can specify the sources
 # this is a required setting
 SOURCES = [
@@ -101,14 +105,15 @@ OUTPUT_DIR = '/data/promnesia'
 '''
 Optional setting.
 A directory to keep intemediate caches in order to speed up indexing.
-If not specified, caching isn't used.
+If not specified, will use user cache directory
+If set to None, cache is disabled
 '''
 CACHE_DIR = '/tmp/promnesia_cache/'
 
 
 '''
 Optional setting.
-Yu can specify the URLs you don't want to be indexed.
+You can specify the URLs you don't want to be indexed.
 You might want it because there are too many of them, or for security/privacy reasons.
 '''
 FILTERS = [
@@ -119,5 +124,16 @@ FILTERS = [
 
     # you can use regexes too!
     'redditmedia.com.*.(jpg|png|gif)',
-
 ]
+
+'''
+Optional setting.
+Can be useful to hack (e.g. rewrite/filter/etc) the visits before inserting in the database.
+'''
+def HOOK(v):
+    # filter out 'boring' github visits without contexts
+    if 'github.com' in v.norm_url:
+        if v.context is None:
+            return
+    # otherwise keep intact
+    yield v
