@@ -1,11 +1,11 @@
 /* @flow */
-import {Visits, Visit, unwrap, format_duration, Methods, addStyle} from './common';
-import type {JsonObject, Second} from './common';
-import {get_options_async} from './options';
+import {Visits, Visit, unwrap, format_duration, Methods, addStyle, Ids} from './common'
+import type {Second} from './common'
+import {getOptions, USE_ORIGINAL_TZ, GROUP_CONSECUTIVE_SECONDS} from './options';
 import type {Options} from './options';
-import {Binder, _fmt} from './display';
+import {Binder, _fmt, asClass} from './display';
 import {defensify} from './notifications';
-import {chromeRuntimeSendMessage} from './async_chrome';
+import {achrome} from './async_chrome'
 
 // TODO how to prevent sidebar hiding on click??
 
@@ -19,6 +19,17 @@ function get_or_default(obj, key, def) {
 const SIDEBAR_ID   = 'promnesia-sidebar';
 const CONTAINER_ID = 'promnesia-sidebar-container';
 
+
+const Cls = {
+    /* marks the highlighted block of text */
+    HIGHLIGHT: 'promnesia-highlight',
+    /* points to the visit/context that caused the highlight */
+    HIGHLIGHT_REF: 'promnesia-highlight-reference',
+    /* just a wrapper trick to force it not take any space */
+    HIGHLIGHT_REF_WRAPPER: 'promnesia-highlight-reference-wrapper',
+}
+
+
 const SIDEBAR_ACTIVE = 'promnesia';
 
 const doc = document;
@@ -63,21 +74,21 @@ class Sidebar {
         // makes it much easier for settings
         cbody.id = SIDEBAR_ID;
         {
-            const show_dots = cdoc.createElement('button');
-            show_dots.appendChild(cdoc.createTextNode('Mark visited'));
+            const show_dots_button = cdoc.createElement('button');
+            show_dots_button.appendChild(cdoc.createTextNode('Mark visited'));
             // TODO hmm. not sure if defensify is gonna work from here? no access to notifications api?
-            show_dots.addEventListener('click', defensify(async () => {
-                await chromeRuntimeSendMessage({method: Methods.MARK_VISITED});
+            show_dots_button.addEventListener('click', defensify(async () => {
+                await achrome.runtime.sendMessage({method: Methods.MARK_VISITED});
             }, 'mark_visited.onClick'));
             // TODO maybe highlight or just use custom class for that?
-            show_dots.title = "Mark visited links on the current page with dots";
-            cbody.appendChild(show_dots);
+            show_dots_button.title = "Mark visited links on the current page with dots";
+            cbody.appendChild(show_dots_button);
         }
         {
             const searchb = cdoc.createElement('button');
             searchb.appendChild(cdoc.createTextNode('Search'));
             searchb.addEventListener('click', defensify(async () => {
-                await chromeRuntimeSendMessage({method: Methods.OPEN_SEARCH});
+                await achrome.runtime.sendMessage({method: Methods.OPEN_SEARCH});
             }, 'open_search.onClick'));
             cbody.appendChild(searchb);
         }
@@ -128,6 +139,7 @@ class Sidebar {
     }
 
     async show() {
+        // NOTE: this is idempotent and should be this way
         const frame = await this.ensureFrame();
         this.body.classList.add(SIDEBAR_ACTIVE);
         frame.style.display = 'block';
@@ -165,7 +177,7 @@ class Sidebar {
         this.setupFrame(sidebar);
 
         // TODO a bit nasty, but at the moment easiest way to solve it
-        // apparetnly iframe is loading about:blank 
+        // apparently iframe is loading about:blank
 
 
         // TODO perhaps better move it to toggle? although maybe not necessary at all
@@ -181,131 +193,197 @@ class Sidebar {
 }
 
 async function sidebar(): Promise<Sidebar> {
-    const opts = await get_options_async();
-    return new Sidebar(opts);
+    const opts = await getOptions()
+    return new Sidebar(opts)
 }
 
-async function toggleSidebar() {
-    (await sidebar()).toggle();
+
+function _sanitize(text: string): string {
+    /* cleanup to make matches more robust */
+    // eslint-disable-next-line no-useless-escape
+    return text.replace(/[.,\/#!$%\^&\*;:{}=\-_`~()]/g,'')
+               .replace(/\s\s+/g, ' ')
+               .trim();
 }
-// to make function available for executeScript... gross
-window.toggleSidebar = toggleSidebar;
 
-
-function findText(elem: Node, text: string): ?Node {
-    let res = null;
-    // TODO need to match or something..
-    // TODO not sure if better to go bottom up?
-    if (elem.textContent.includes(text)) {
-        res = elem;
-    }
-    const children = elem.childNodes;
-    if (children == null) {
-        return res;
-    }
-    for (let x of children) {
-        let cr = findText(x, text);
-        if (cr != null) {
-            return cr;
+// yields all DOM nodes that matched against some of the lines
+function* findMatches(elem: Node, lines: Set<string>): Iterable<[string, Node]> {
+    // first try to highlight the most specific element possible
+    const children = elem.childNodes || []
+    let found = false
+    for (let c of children) {
+        for (const m of findMatches(c, lines)) {
+            found = true
+            yield m
         }
     }
-    return res;
+    if (found) {
+        // some of the children matched.. so no need to process the element itself?
+        return
+    }
+    // todo would be nice to cache it between highlight calls?
+    // maybe just set on the element?
+    const stext = _sanitize(elem.textContent)
+    let matched = null
+    for (const line of lines) {
+        if (stext.includes(line)) {
+            matched = line // make sure we break from the loop first so the iterator isn't invalidated
+            break
+        }
+    }
+    if (matched != null) {
+        // no need to process it further
+        lines.delete(matched)
+        yield [matched, elem]
+    }
 }
 
-// TODO not very effecient; replace with something existing (Hypothesis??)
-function _highlight(text: string, idx: number) {
+// TODO potentially not very effecient; replace with something existing (Hypothesis??)
+function _highlight(text: string, idx: number, v: Visit) {
+    const lines = new Set()
     for (const line of text.split('\n')) {
-        // TODO filter too short strings? or maybe only pick the longest one?
-        const found = findText(unwrap(doc.body), line);
-        if (found == null) {
-            console.debug('No match found for %s', line);
-            continue;
+        let sline = line.trim()
+        if (sline.length == 0) {
+            continue // no need to log
         }
-        console.debug("highlighting %o %s", found, line);
+        sline = _sanitize(line)
+        if (sline.length <= 3) {
+            console.debug("promnesia: line '%s' was completely sanitized/too short.. skipping", line)
+            continue
+        }
+        lines.add(sline)
+    }
 
-        // $FlowFixMe
-        const target: HTMLElement = unwrap(found.nodeType == Node.TEXT_NODE ? found.parentElement : found);
+    // TODO make sure they are unique? so we don't hl the same element twice..
+    const to_hl = []
+    for (let [line, target] of findMatches(unwrap(doc.body), lines)) {
+        // https://developer.mozilla.org/en-US/docs/Web/API/Node/nodeType#Node_type_constants
+        while (target != null && target.nodeType != Node.ELEMENT_NODE) {
+            target = target.parentElement
+        }
+
+        if (target == null) {
+            // eh. only text nodes are present, can't attach highlight to anything
+            continue
+        }
+        target = ((target: any): HTMLElement)
+
+        if (target.classList.contains(Cls.HIGHLIGHT)) {
+            continue // shouldn't act on self
+        }
 
         if (target.classList.contains('toastify')) {
-            // TODO hacky...
-            continue;
+            // hacky.. to avoid a race condition when toast notification is shown
+            continue
         }
 
         // TODO why doesn't flow warn about this??
         // target.name === 'body'
         if (target === doc.body) {
             // meh, but otherwise too spammy
-            console.warn('body matched for highlight; skipping it');
+            console.debug('promnesia: body matched for highlight; skipping it')
             continue;
         }
+        const d = unwrap(document.documentElement)
+        const rect = target.getBoundingClientRect()
+        const ratio = (rect.width * rect.height) / (d.scrollWidth * d.scrollHeight)
+        const RATIO = 0.5 // kinda arbitrary
+        if (ratio > RATIO) {
+            console.warn('promnesia: matched element %o is too big (ratio %f > %f). skipping it', target, ratio, RATIO)
+            continue
+        }
+        console.debug("promnesia: '%s': matched %o", line, target)
+
+        // defer changing DOM to avoid reflow? not sure if actually an issue but just in case..
+        // https://gist.github.com/paulirish/5d52fb081b3570c81e3a
+        to_hl.push(target)
+    }
+    for (const target of to_hl) {
+        // NOTE: there is <mark> tag, but it doesn't do anything apart from
+        // so perhaps best to keep the DOM intact
+        target.classList.add(Cls.HIGHLIGHT)
+
+        const refc = doc.createElement('span')
+        refc.classList.add(Cls.HIGHLIGHT_REF_WRAPPER)
+        refc.classList.add('nonselectable')
+        refc.style.width  = '0px'
+        refc.style.height = '0px'
+        refc.style.position = 'absolute'
 
-        target.classList.add('promnesia-highlight');
         const ref = doc.createElement('span');
-        ref.classList.add('promnesia-highlight-reference');
-        ref.classList.add('nonselectable');
-        ref.appendChild(doc.createTextNode(String(idx)));
-        target.insertAdjacentElement('beforeend', ref);
+        ref.classList.add(Cls.HIGHLIGHT_REF);
+        ref.textContent = String(idx)
+        ref.title = `promnesia #${idx}: ${v.tags.join(' ')} ${(USE_ORIGINAL_TZ ? v.dt_local : v.time).toLocaleString()}`
+        ref.style.position = 'relative'
+        refc.appendChild(ref)
+
+        target.insertAdjacentElement('beforeend', refc);
     }
 }
 
-function tryHighlight(text: string, idx: number) {
-    // TODO sidebar could also display if highlight matched or if it's "orphaned"
+function tryHighlight(text: string, idx: number, v: Visit) {
+    // todo sidebar could also display if highlight matched or if it's "orphaned"
     // TODO use tag color for background?
     try {
-        _highlight(text, idx);
+        _highlight(text, idx, v)
     } catch (error) {
-        console.error('Error while highlighting %s: %o', text, error); // TODO come up with something better..
+        console.error('promnesia: while highlighting %s %o: %o', text, v, error)
     }
 }
 
-// used dynamically
-// eslint-disable-next-line no-unused-vars
-async function bindError(message: string) {
-    const opts = await get_options_async();
-    const sidebar = new Sidebar(opts);
 
-    const cont = await sidebar.getContainer();
-    await sidebar.clear(); // TODO probably, unnecessary?
-
-    const binder = new Binder(doc);
-    binder.error(cont, message);
-}
-
-
-// TODO rename to 'set'?
-async function bindSidebarData(response: JsonObject) {
+/**
+ * yields UI updates, so it's possible to schedule them in blocks and avoid blocking
+ */
+// TODO ugh. how to profile it? how to make sure it can never slow down user UI?
+async function* _bindSidebarData(response: Visits) {
     // TODO ugh. we probably only want to set data, don't want to do anything with dom until we trigger the sidebar?
     // TDO perhaps we need something reactive...
     // window.sidebarData = response;
-    const opts = await get_options_async();
+    const opts = await getOptions();
     const sidebar = new Sidebar(opts);
 
     const cont = await sidebar.getContainer();
     await sidebar.clear(); // TODO probably, unnecessary?
 
-    const binder = new Binder(doc);
+    // a way to ensure the sidebar rendering isn't impacting the browsing experience
+    // TODO use the same logic in search?
+    const YIELD_DELAY_MS = 200
+    let last_yield = 0
+    let iterations_since = 0
+    function shouldYield() {
+        let cur = new Date().getTime()
+        iterations_since += 1
+        const took = cur - last_yield
+        if (took < YIELD_DELAY_MS) {
+            return false
+        }
+        // todo eh, would be nice to log something more useful here..
+        console.debug('promnesia: bindSidebarData: %d iterations passed, took %o ms', iterations_since, took)
+        last_yield = cur
+        iterations_since = 0
+        return true
+    }
+    //
+
+    const binder = new Binder(doc, opts)
 
     const all_tags_c = binder.makeChild(cont, 'div', ['src-filter']);
     const items = binder.makeChild(cont, 'ul');
-    items.id = 'visits';
+    items.id = Ids.VISITS
 
+    const [visits, errors] = response.partition()
+    const normalised = response.normalised_url
+
+    for (const err of errors) {
+        await binder.renderError(items, err)
+    }
 
-    // TODO why has this ended up serialised??
-    const visits = response.visits.map(rvisit =>
-        new Visit(
-            rvisit.original_url,
-            rvisit.normalised_url,
-            new Date(rvisit.time), // TODO careful about utc here?
-            rvisit.tags,
-            rvisit.context,
-            rvisit.locator,
-            rvisit.duration,
-        ) // TODO ugh ugly..
-    );
     visits.sort((f, s) => {
         // keep 'relatives' in the bottom
-        const fr = f.normalised_url === response.normalised_url;
-        const sr = s.normalised_url === response.normalised_url;
+        // TODO: this might slightly break local visits sorting, becuase they don't necessarily have proper normalisation
+        const fr = f.normalised_url === normalised
+        const sr = s.normalised_url === normalised
         if (fr != sr) {
             return (sr ? 1 : 0) - (fr ? 1 : 0);
         }
@@ -337,6 +415,8 @@ async function bindSidebarData(response: JsonObject) {
     for (let [tag, count] of [[null, with_ctx.length], ...Array.from(all_tags).sort()]) {
         let predicate: ((string) => boolean);
         if (tag === null) {
+            // TODO https://github.com/karlicoss/promnesia/issues/132
+            // maybe just reorder them and show after visits with context?
             // meh
             tag = 'all';
             predicate = () => true;
@@ -346,32 +426,43 @@ async function bindSidebarData(response: JsonObject) {
 
         // TODO show total counts?
         // TODO if too many tags, just overlap on the seconds line
-        const tag_c = binder.makeChild(all_tags_c, 'span', ['src', tag]);
+        const tag_c = binder.makeChild(all_tags_c, 'span', ['src', asClass(tag)])
         binder.makeTchild(tag_c, `${tag} (${count})`);
         // TODO checkbox??
         tag_c.addEventListener('click', () => {
             for (const x of items.children) {
-                const sources = unwrap(x.getAttribute('sources')).split(' ');
+                const sources = unwrap(x.dataset['sources']).split(' ');
                 const found = sources.some(predicate);
                 x.style.display = found ? 'block' : 'none';
             }
         });
     }
 
+    const visit_date_time = (v: Visit) => _fmt(USE_ORIGINAL_TZ ? v.dt_local : v.time)
 
-    for (const [idx0, v] of with_ctx.entries()) {
+    const entries = with_ctx.entries() // NOTE: it's an iterator
+    /*
+     * ugh. hacky way to defer UI thread updating... otherwise it blocks the interface on too many visits
+     * TBH this is a beat stupid... surely async style execution would be super useful for this, why is it not here by default?
+     */
+    for (const [idx0, v] of entries) {
+        if (shouldYield()) {
+            yield
+        }
         const idx1 = idx0 + 1; // eh, I guess that makes more sense for humans
         const ctx = unwrap(v.context);
 
         // TODO hmm. hopefully chrome visits wouldn't get highlighted here?
-        const relative = v.normalised_url != response.normalised_url;
+        const relative = v.normalised_url != normalised
 
         if (!relative && opts.highlight_on) {
-            tryHighlight(ctx, idx1);
+            // todo this might compete for execution with the sidebar rendering...
+            // later maybe integrate it in the yield mechanism..
+            setTimeout(() => tryHighlight(ctx, idx1, v))
         }
 
-
-        const [dates, times] = _fmt(v.time);
+        // TODO maybe shouldn't attach immediately? not sure
+        const [dates, times] = visit_date_time(v)
         binder.render(items, dates, times, v.tags, {
             idx           : idx1,
             timestamp     : v.time,
@@ -380,38 +471,40 @@ async function bindSidebarData(response: JsonObject) {
             context       : v.context,
             locator       : v.locator,
             relative      : relative,
-        });
+        })
     }
 
-
+    const delta_ms = GROUP_CONSECUTIVE_SECONDS * 1000;
     function* groups() {
         let group = [];
-        const delta = 20 * 60 * 1000;
         for (const v of no_ctx) {
             const last = group.length == 0 ? v : group[group.length - 1];
-            if (last.time - v.time > delta) {
+            const diff = last.time - v.time
+            if (diff > delta_ms) {
                 if (group.length > 0) {
                     yield group;
                 }
-                group.length = 0
+                group = []
             }
             group.push(v);
         }
         if (group.length > 0) {
             yield group;
         }
-        group.length = 0
     }
 
-    const tag_map = opts.src_map;
-    // TODO group ones with no ctx..
+    // todo maybe it should be a generic hook instead?
+    // todo how to make it defensive?..
+    const tag_map = JSON.parse(opts.src_map)
     for (const group of groups()) {
-
+        if (shouldYield()) {
+            yield // give way to UI thread
+        }
         const first = group[0];
         const last  = group[group.length - 1];
         // eslint-disable-next-line no-unused-vars
-        const [fdates, ftimes] = _fmt(first.time);
-        const [ldates, ltimes] = _fmt(last.time);
+        const [fdates, ftimes] = visit_date_time(first)
+        const [ldates, ltimes] = visit_date_time(last)
         const dates = ldates;
         const times = ltimes == ftimes ? ltimes : ltimes + "-" + ftimes;
         const tset = new Set();
@@ -430,7 +523,7 @@ async function bindSidebarData(response: JsonObject) {
         }
         const tags = [...tset].sort();
         const ctx = total_dur == null ? null : `Time spent: ${format_duration(total_dur)}`;
-        binder.render(items, dates, times, tags, {
+        await binder.render(items, dates, times, tags, {
             idx: null,
             timestamp: first.time,
             original_url  : null,
@@ -442,24 +535,48 @@ async function bindSidebarData(response: JsonObject) {
     }
 }
 
+async function bindSidebarData(response: Visits) {
+    const dom_updates_gen = _bindSidebarData(response)
+    async function consume_one() {
+        // consume head
+        const res = await dom_updates_gen.next()
+        if (!res.done) {
+            // schedule tail
+            setTimeout(consume_one)
+        }
+    }
+    await consume_one()
+}
+
 
-// hmm. otherwise it can't be called from executescript??
-window.bindSidebarData = bindSidebarData;
-window.bindError       = bindError;
 
 // TODO ugh, it actually seems to erase all the class information :( is it due to message passing??
 // eslint-disable-next-line no-unused-vars
-function requestVisits() {
-    chromeRuntimeSendMessage({method: Methods.GET_SIDEBAR_VISITS})
-        .then((response: ?Visits) => {
-            if (response == null) {
-                return;
-            }
-            bindSidebarData(response);
-        });
+function requestVisits(): void {
+    achrome.runtime.sendMessage({method: Methods.GET_SIDEBAR_VISITS})
+           .then((response: {}) => {
+               if (response == null) {
+                   // todo why would it be?
+                   return
+               }
+               bindSidebarData(Visits.fromJObject(response))
+           })
 }
 
 
+// eslint-disable-next-line no-unused-vars
+chrome.runtime.onMessage.addListener((msg: any, sender: chrome$MessageSender) => {
+    const method = msg.method
+    if        (method == Methods.BIND_SIDEBAR_VISITS) {
+        bindSidebarData(Visits.fromJObject(msg.data))
+    } else if (method == Methods.SIDEBAR_SHOW) {
+        sidebar().then(s => s.show())
+    } else if (method == Methods.SIDEBAR_TOGGLE) {
+        sidebar().then(s => s.toggle())
+    }
+    // todo do I need to return anything?
+})
+
 // TODO make configurable? or resizable?
 
 // TODO shit. if it's not an iframe can't be scrollable?
