@@ -36,14 +36,14 @@ const modifier = target === T.CHROME ? 'Shift' : 'Alt';
 // ugh. declarative formats are shit.
 const commandsExtra = {
     "_execute_browser_action": {
-        "description": "Activate extension",
+        "description": "Activate sidebar",
         "suggested_key": {
             "default": `Ctrl+${modifier}+E`,
             "mac":  `Command+${modifier}+E`
         }
     },
     "mark_visited": {
-        "description": "Mark visited link on the current page",
+        "description": "Mark/unmark visited links on the current page",
         "suggested_key": {
             "default": `Ctrl+${modifier}+V`,
             "mac":  `Command+${modifier}+V`
@@ -65,7 +65,7 @@ const commandsExtra = {
 // TODO ugh it's getting messy...
 const action = {
     "default_icon": "images/ic_not_visited_48.png",
-    "default_title": "Was not visited",
+    "default_title": "Show promnesia sidebar",
 };
 
 
@@ -86,6 +86,12 @@ const manifestExtra = {
     browser_action: action,
     permissions: permissionsExtra,
     options_ui: {},
+    web_accessible_resources: [
+        // wtf?? it says that content scritps don't need to be listed... but doesn't work otherwise..
+        // https://developer.mozilla.org/en-US/docs/Mozilla/Add-ons/WebExtensions/manifest.json/web_accessible_resources#Description
+        "vendors_anchorme.js",
+        // TODO however, seems that web_accessible_resources works without sidebar.js and sidebar.css?? odd
+    ],
 };
 
 if (dev) {
@@ -162,9 +168,16 @@ const options = {
   output: {
     path: buildPath,
     filename: '[name].js',
+    // chunkFilename: '[name].bundle.js',
+  },
+  optimization: {
+    splitChunks: {
+      automaticNameDelimiter: '_', // ugh. default ~ can't be loaded by the browser??
+    }
   },
   module: {
-      rules: [{
+      rules: [
+      {
           test: /\.js$/,
           exclude: /node_modules/,
           use: {
@@ -179,6 +192,37 @@ const options = {
           test: /\.html$/,
           loader: 'html-loader',
           exclude: /node_modules/
+      },
+      {
+          test: /node_modules\/basket.js\/lib\/basket.js/,
+          use: {
+              loader: path.join(__dirname, 'patcher.js'),
+              options: {
+                  patches: [
+                      /* see
+                       * https://github.com/addyosmani/basket.js/issues/174
+                       * https://github.com/addyosmani/basket.js/issues/173
+                       */
+                      {code: /RSVP.Promise/g, newCode: 'Promise'},
+                      {code: 'RSVP.all', newCode: 'Promise.all'},
+                      {code: 'window.basket = {', newCode: 'var basket = window.basket= {'},
+                  ],
+              }
+          },
+      },
+      {
+          test: /node_modules\/anchorme\/dist\/browser\/anchorme.js/,
+          use: {
+              loader: path.join(__dirname, 'patcher.js'),
+              options: {
+                  patches: [
+                      /* need [] for org-mode */
+                      // https://github.com/alexcorvi/anchorme.js/blob/098843bc0d042601cff592c4f8c9f6d0424c09cd/src/regex.ts#L11
+                      {code   : '\\\\[\\\\]', // need double escaping for \...
+                       newCode: ''          } // TODO ugh. sort of flaky..
+                  ],
+              }
+          },
       }
     ]
   },
@@ -186,12 +230,10 @@ const options = {
    new CleanWebpackPlugin(), // ok, respects symlinks
    new CopyWebpackPlugin([
       { from: 'images/*' },
-       // TODO compress maybe...
-      { from: 'shallalist/finance/banking/domains', to: 'shallalist/finance/banking' },
-      { from: 'shallalist/webmail/domains'        , to: 'shallalist/webmail' },
       { from: 'src/*.html'     , flatten: true},
       { from: 'src/*.css'      , flatten: true},
       { from: 'src/toastify.js', flatten: true}, // TODO my version is tweaked, right?
+      { from: 'src/showvisited.js', flatten: true},
       // { from: 'node_modules/webextension-polyfill/dist/browser-polyfill.min.js'},
     ]),
     new WebpackExtensionManifestPlugin({
