@@ -11,9 +11,10 @@ Also some experiments to establish 'URL hierarchy'.
 """
 # TODO eh?? they fixed mobile.twitter.com?
 
+from itertools import chain
 import re
 import typing
-from typing import Iterable, NamedTuple, Set, Optional, List, Sequence, Union, Tuple, Dict
+from typing import Iterable, NamedTuple, Set, Optional, List, Sequence, Union, Tuple, Dict, Any, Collection
 
 import urllib.parse
 from urllib.parse import urlsplit, parse_qsl, urlunsplit, parse_qs, urlencode, SplitResult
@@ -38,6 +39,7 @@ def try_cutr(suffix, s):
     else:
         return s
 
+# TODO move this to site-specific normalisers?
 dom_subst = [
     ('m.youtube.'     , 'youtube.'),
     ('studio.youtube.', 'youtube.'),
@@ -88,41 +90,42 @@ default_qremove = {
     'utf8',
 }
 
-default_qkeep = {
+default_qkeep = [
     # ok, various BBS have it, hackernews has it etc?
     # hopefully it's a reasonable one to keep..
     'id',
-}
+
+    # common to forums, usually means 'thread'
+    't',
+
+    # common to some sites.., usually 'post'
+    'p',
+]
 
 # TODO perhaps, decide if fragment is meaningful (e.g. wiki) or random sequence of letters?
 class Spec(NamedTuple):
-    qkeep  : Optional[Set[str]] = None
+    qkeep  : Optional[Collection[str]] = None
     qremove: Optional[Set[str]] = None
     fkeep  : bool = False
 
-    def keep_query(self, q: str) -> bool:
-        qkeep   = default_qkeep  .union(self.qkeep or {})
+    def keep_query(self, q: str) -> Optional[int]: # returns order
+        qkeep = {
+            q: i for i, q in enumerate(chain(default_qkeep, self.qkeep or []))
+        }
         qremove = default_qremove.union(self.qremove or {})
         # I suppose 'remove' is only useful for logging. we remove by default anyway
 
         keep = False
         remove = False
-        if qkeep is not None and q in qkeep:
-            keep = True
-        # pylint: disable=unsupported-membership-test
+        qk = qkeep.get(q)
+        if qk is not None:
+            return qk
+        # todo later, check if spec tells both to keep and remove?
         if q in qremove:
-            remove = True
-        if keep and remove:
-            # keep wins?
-            return True # TODO need a warning
-        if keep:
-            return True
-        if remove:
-            return False
-
+            return None
         # by default drop all
         # it's a better default, since if it's *too* unified, the user would notice it. but not vice versa!
-        return False
+        return None
 
     @classmethod
     def make(cls, **kwargs):
@@ -134,14 +137,13 @@ S = Spec.make
 specs = {
     'youtube.com': S(
         # TODO search_query?
-        qkeep={
-            'list', # TODO hmm. list is kinda important for playlist urls...
+        qkeep=[ # note: experimental.. order matters here
             'v',
-        }, # TODO frozenset?
+            't',
+            'list',
+        ],
+        # todo frozenset?
         qremove={
-            't', # TODO not so sure about it
-
-
             'time_continue', 'index', 'feature', 'lc', 'app', 'start_radio', 'pbjreload', 'annotation_id',
             'flow', 'sort', 'view',
             'enablejsapi', 'wmode', 'html5', 'autoplay', 'ar',
@@ -177,10 +179,9 @@ specs = {
     'wikipedia.org': S(fkeep=True),
     'scottaaronson.com'  : S(qkeep={'p'}, fkeep=True),
     'urbandictionary.com': S(qkeep={'term'}),
-    'ycombinator.com'    : S(qkeep={'id'}),
+    'ycombinator.com'    : S(qkeep={'id'}), # todo just keep id by default?
     'play.google.com'    : S(qkeep={'id'}),
     'answers.yahoo.com'  : S(qkeep={'qid'}),
-    'ubuntuforums.org'   : S(qkeep={'t'}),
 }
 
 _def_spec = S()
@@ -199,6 +200,37 @@ def get_spec(dom: str) -> Spec:
             return sp
     return _def_spec
 
+
+
+# ideally we'd just be able to reference the domain name and use it in the subst?
+# some sort of hierarchical matchers? not sure what's got better performance..
+# if 'from?site=' in url:
+#     return p.query('site')
+
+Spec2 = Any # TODO
+
+# TODO this should be a map
+Frag = Any
+Parts = Sequence[Tuple[str, str]]
+
+
+def _yc(domain, path, qq: Parts, frag: Frag) -> Tuple[Any, Any, Parts, Frag]:
+    if path[:5] == '/from':
+        site = dict(qq).get('site')
+        if site is not None:
+            domain = site
+            path = ''
+            qq = ()
+            frag = ''
+    # TODO this should be in-place? for brevity?
+    return (domain, path, qq, frag)
+
+def get_spec2(dom: str) -> Optional[Spec2]:
+    return {
+        'news.ycombinator.com': _yc,
+    }.get(dom)
+
+
 class CanonifyException(Exception):
     pass
 
@@ -287,6 +319,7 @@ def transform_split(split: SplitResult):
 
         (netloc, path, qq) = [t.format(**gd) for t in to]
         qparts.extend(parse_qsl(qq)) # TODO hacky..
+        # TODO eh, qparts should really be a map or something...
         break
 
 
@@ -362,17 +395,27 @@ def canonify(url: str) -> str:
 
     domain, path, qq, _frag = transform_split(parts)
 
+    spec2 = get_spec2(domain)
+    if spec2 is not None:
+        # meh
+        domain, path, qq, _frag = spec2(domain, path, qq, _frag)
+
+
     spec = get_spec(domain)
 
     # TODO FIXME turn this logic back on?
     # frag = parts.fragment if spec.fkeep else ''
     frag = ''
 
-    qq = [(k, v) for k, v in qq if spec.keep_query(k)]
-    qq = list(sorted(qq)) # order matters in theory, but definitely not by default
+    iqq = []
+    for k, v in qq:
+        order = spec.keep_query(k)
+        if order is not None:
+            iqq.append((order, k, v))
+    qq = [(k, v) for i, k, v in sorted(iqq)]
     # TODO still not sure what we should do..
     # quote_plus replaces %20 with +, not sure if we want it...
-    query = urlencode(qq, quote_via=quote_via)
+    query = urlencode(qq, quote_via=quote_via) # type: ignore[type-var]
 
     path = _quote_path(path)
 
@@ -579,6 +622,8 @@ FB_PATTERNS = [
     r'F/notes/U/P',
 ]
 
+
+# NOTE: right, I think this is just for analysis so far... not actually used
 PATTERNS = {
     'twitter'   : TW_PATTERNS,
     'reddit'    : RD_PATTERNS,
@@ -587,6 +632,7 @@ PATTERNS = {
     'stackoverflow': SO_PATTERNS,
     'facebook'  : FB_PATTERNS,
     'wikipedia' : WK_PATTERNS,
+    # 'news.ycombinator.com': YC_PATTERNS,
 }
 
 
