@@ -1,18 +1,17 @@
 from collections.abc import Sized
 from datetime import datetime, date
 import os
-import re
 from typing import NamedTuple, Set, Iterable, Dict, TypeVar, Callable, List, Optional, Union, Any, Collection, Sequence, Tuple, TypeVar
 from pathlib import Path
 from glob import glob
 import itertools
 import logging
 from functools import lru_cache
-import traceback
+import shutil
 import pytz
 import warnings
 
-from .cannon import CanonifyException, canonify
+from .cannon import canonify
 
 
 T = TypeVar('T')
@@ -36,15 +35,32 @@ class Loc(NamedTuple):
         return cls(title=title, href=href)
 
     @classmethod
-    def file(cls, path: PathIsh, line: Optional[int]=None):
-        ll = '' if line is None else f':{line}'
+    def file(cls, path: PathIsh, line: Optional[int]=None, relative_to: Optional[Path]=None):
+        lstr = '' if line is None else f':{line}'
         # todo loc should be url encoded? dunno.
         # or use line=? eh. I don't know. Just ask in issues.
-        loc = f'{path}{ll}'
+
+        # todo: handler has to be overridable by config. This is needed for docker, but also for a "as a service" install, where the sources would be available on some remote webserver
+        # maybe it should be treated as a format string, so that {line} may be a part of the result or not.
+        # for local usage, editor:///file:line works, but if the txt file is only avaiable through http, it breaks.
+        #if get_config().MIME_HANDLER:
+        #   handler = get_config().MIME_HANDLER
+        #if True:
+        #    handler =  'editor:///home/koom/promnesia/docker/'
+        #else:
         handler = _detect_mime_handler()
+
+        rel = Path(path)
+        if relative_to is not None:
+            try:
+                # making it relative is a bit nicer for display
+                rel = rel.relative_to(relative_to)
+            except Exception as e:
+                pass # todo log/warn?
+        loc = f'{rel}{lstr}'
         return cls.make(
             title=loc,
-            href=f'{handler}{loc}'
+            href=f'{handler}{path}{lstr}'
         )
 
     # TODO need some uniform way of string conversion
@@ -83,7 +99,7 @@ def _detect_mime_handler() -> str:
     if not exists('editor'):
         warnings.warn('''
         You might want to install https://github.com/karlicoss/open-in-editor
-        So you can jump you your text files straight from the browser
+        So you can jump to your text files straight from the browser
 '''.rstrip())
     else:
         result = 'editor://'
@@ -153,111 +169,13 @@ class DbVisit(NamedTuple):
 
 Filter = Callable[[Url], bool]
 
-def make_filter(thing) -> Filter:
-    if isinstance(thing, str):
-        rc = re.compile(thing)
-        def filter_(u: str) -> bool:
-            return rc.search(u) is not None
-        return filter_
-    else: # must be predicate
-        return thing
 
+from .logging import LazyLogger
+logger = LazyLogger('promnesia', level='DEBUG')
 
 def get_logger() -> logging.Logger:
-    return logging.getLogger("promnesia")
+    return logger
 
-# TODO do i really need to inherit this??
-class History(Sized):
-    # TODO I guess instead filter on DbVisit making site?
-    FILTERS: List[Filter] = [
-        make_filter(f) for f in
-        [
-            r'^chrome-devtools://',
-            r'^chrome-extension://',
-            r'^chrome-error://',
-            r'^chrome-native://',
-            r'^chrome-search://',
-
-            r'chrome://newtab',
-            r'chrome://apps',
-            r'chrome://history',
-
-            r'^about:',
-            r'^blob:',
-            r'^view-source:',
-
-            r'^content:',
-
-            # TODO maybe file:// too?
-            # chrome-search:
-        ]
-    ]
-
-    @classmethod
-    def add_filter(cls, filterish):
-        cls.FILTERS.append(make_filter(filterish))
-
-    def __init__(self, *, src: SourceName):
-        self.vmap: Dict[Visit, DbVisit] = {}
-        # TODO err... why does it map from previsit???
-        self.logger = get_logger()
-        self.src = src
-
-    # TODO mm. maybe history should get filters from some global config?
-    # wonder how okay is it to set class attribute..
-
-    @classmethod
-    def filtered(cls, url: Url) -> bool:
-        for f in cls.FILTERS:
-            if f(url):
-                return True
-        return False
-
-    @property
-    def visits(self) -> List[DbVisit]:
-        return list(self.vmap.values())
-
-    def register(self, v: Visit) -> Optional[Exception]:
-        # TODO should we filter before normalising? not sure...
-        if History.filtered(v.url):
-            return None
-
-        # TODO perhaps take normalised into account here??
-        if v in self.vmap:
-            return None
-
-        res = DbVisit.make(v, src=self.src)
-        if isinstance(res, CanonifyException):
-            self.logger.error('error while canonnifying %s... ignoring', v)
-            self.logger.exception(res)
-            return None
-        elif isinstance(res, Exception):
-            return res
-        else:
-            db_visit = res
-
-        self.vmap[v] = db_visit
-        return None
-        # TODO hmm some filters make sense before stripping off protocol...
-
-    ## only used in tests?..
-    def _nmap(self):
-        from itertools import groupby
-        key = lambda x: x.norm_url
-        return {k: list(g) for k, g in groupby(sorted(self.visits, key=key), key=key)}
-
-    def __len__(self) -> int:
-        return len(self._nmap())
-
-    def __contains__(self, url) -> bool:
-        return url in self._nmap()
-
-    def __getitem__(self, url: Url):
-        return self._nmap()[url]
-    #
-
-    def __repr__(self):
-        return 'History{' + repr(self.visits) + '}'
 
 
 # kinda singleton
@@ -277,6 +195,7 @@ def _get_extractor(syntax: Syntax):
     u = URLExtract()
     # https://github.com/lipoja/URLExtract/issues/13
     if syntax in {'org', 'orgmode', 'org-mode'}: # TODO remove hardcoding..
+        # handle org-mode links properly..
         u._stop_chars_right |= {'[', ']'}
         u._stop_chars_left  |= {'[', ']'}
     elif syntax in {'md', 'markdown'}:
@@ -286,7 +205,7 @@ def _get_extractor(syntax: Syntax):
     return u
 
 
-def sanitize(url: str) -> str:
+def _sanitize(url: str) -> str:
     # TODO not sure it's a really good idea.. but seems easiest now
     # TODO have whitelisted urls that allow trailing parens??
     url = url.strip(',.…\\')
@@ -295,35 +214,18 @@ def sanitize(url: str) -> str:
         url = url.strip(')')
     return url
 
-# TODO sort just in case? not sure..
-def _extract_line_urls(*, s: str, syntax: Syntax) -> List[str]:
-    # TODO unit test for escaped urls.. or should it be in normalise instead?
-    if len(s.strip()) == 0:
-        return [] # optimize just in case..
 
-    # TODO special handling for org links
-
-    # TODO fuck. doesn't look like it's handling multiple urls in same line well...
-    # ("python.org/one.html python.org/two.html",
-    # hopefully ok... I guess if there are spaces in url we are fucked anyway..
+def iter_urls(s: str, *, syntax: Syntax='') -> Iterable[Url]:
     extractor = _get_extractor(syntax=syntax)
-    urls: List[str] = []
-    for x in s.split():
-        urls.extend(extractor.find_urls(x))
-
-    return [sanitize(u) for u in urls]
+    # note: it also has get_indices, might be usefull
+    for u in extractor.gen_urls(s):
+        yield _sanitize(u)
 
 
-def _extract_urls(*, s: str, syntax: Syntax) -> Iterable[Url]:
-    for line in s.splitlines():
-        yield from _extract_line_urls(s=line, syntax=syntax)
+def extract_urls(s: str, *, syntax: Syntax='') -> List[Url]:
+    return list(iter_urls(s=s, syntax=syntax))
 
 
-def extract_urls(s: str, syntax: Syntax='') -> List[Url]:
-    return list(_extract_urls(s=s, syntax=syntax))
-
-
-# TODO maybe belongs to HPI?
 def from_epoch(ts: int) -> datetime:
     return datetime.fromtimestamp(ts, tz=pytz.utc)
 
@@ -389,53 +291,6 @@ class Source:
 Indexer = Source
 
 
-# TODO do we really need it?
-def previsits_to_history(extractor, *, src: SourceName) -> Tuple[List[DbVisit], List[Exception]]:
-    ex = extractor
-    # TODO isinstance wrapper?
-    # TODO make more defensive?
-    logger = get_logger()
-
-    log_info: str
-    if isinstance(ex, Indexer):
-        log_info = f'{ex.ff.__module__}:{ex.ff.__name__} {ex.args} {ex.kwargs} ...'
-        extr = lambda: ex.ff(*ex.args, **ex.kwargs)
-    else:
-        # TODO if it's a lambda?
-        log_info = f'{ex.__module__}:{ex.__name__}'
-        extr = ex
-
-
-    logger.info('extracting via %s ...', log_info)
-
-    h = History(src=src)
-    errors = []
-    try:
-        previsits = list(extr())
-    except Exception as e:
-        logger.exception(e)
-        return [], [e]
-
-    for p in previsits:
-        if isinstance(p, Exception):
-            errors.append(p)
-            parts = ['indexer emitted exception\n']
-            # eh, exception type is ignored by format_exception completely, apparently??
-            parts.extend(traceback.format_exception(Exception, p, p.__traceback__))
-            logger.error(''.join(parts))
-            continue
-
-        # TODO check whether it's filtered before construction? probably doesn't really impact
-        res = h.register(p)
-        if isinstance(res, Exception):
-            logger.exception(res)
-            errors.append(res)
-
-    # TODO should handle filtering properly?
-    logger.info('extracting via %s: got %d visits', log_info, len(h))
-    return h.visits, errors
-
-
 # not sure if necessary anymore?
 # NOTE: used in configs...
 def last(path: PathIsh, *parts: str) -> Path:
@@ -444,11 +299,18 @@ def last(path: PathIsh, *parts: str) -> Path:
     return Path(max(glob(pp, recursive=True)))
 
 
-from .kython.klogging2 import setup_logger
+from .logging import setup_logger
 
+from copy import copy
 def echain(ex: Exception, cause: Exception) -> Exception:
-    ex.__cause__ = cause
-    return ex
+    e = copy(ex)
+    e.__cause__ = cause
+    # right.. even if we attach cause it doesn't help much because when we return/yield exception, we lose the stacktrace
+    # so only 'ex' gets logged, cause is completely lost
+    # hopefully this is safe? at least on runtimeerrors
+    # might also do something smarter and collapes if both are strings or something..
+    e.args += cause.args
+    return e
 
 
 def slugify(x: str) -> str:
@@ -468,7 +330,139 @@ def appdirs():
 
 
 def default_output_dir() -> Path:
+    # TODO: on Windows, there are two extra subdirectories (<AppAuthor>\<AppName>)
+    # perhaps makes sense to create it here with parents to avoid issues downstream?
     return Path(appdirs().user_data_dir)
 
-# TODO deprecate..
-PreVisit = Visit
+
+def default_cache_dir() -> Path:
+    return Path(appdirs().user_cache_dir)
+
+
+# make it lazy, otherwise it might crash on module import (e.g. on Windows)
+# ideally would be nice to fix it properly https://github.com/ahupp/python-magic#windows
+@lru_cache(1)
+def _magic():
+    logger = get_logger()
+    try:
+        import magic # type: ignore
+    except ModuleNotFoundError as me:
+        logger.exception(me)
+        msg = "python-magic is not detected. It's recommended for better file type detection (pip3 install --user python-magic). See https://github.com/ahupp/python-magic#installation"
+        logger.warning(msg)
+        warnings.warn(msg)
+        return lambda path: None # stub
+    else:
+        mm = magic.Magic(mime=True)
+        return mm.from_file
+
+
+@lru_cache(1)
+def _mimetypes():
+    import mimetypes
+    mimetypes.init()
+    return mimetypes
+
+
+def mime(path: PathIsh) -> Optional[str]:
+    ps = str(path)
+    mimetypes = _mimetypes()
+    # first try mimetypes, it's only using the filename without opening the file
+    pm, _ = mimetypes.guess_type(ps)
+    if pm is not None:
+        return pm
+    # next, libmagic, it might access the file, so a bit slower
+    magic = _magic()
+    return magic(ps)
+
+
+def find_args(root: Path, follow: bool) -> List[str]:
+    return [
+        *(['-L'] if follow else []),
+        str(root),
+        '-type', 'f',
+    ]
+
+
+def fdfind_args(root: Path, follow: bool) -> List[str]:
+    from .config import extra_fd_args
+    return [
+        *extra_fd_args(),
+        *(['--follow'] if follow else []),
+        '--type', 'f',
+        '.',
+        str(root),
+    ]
+
+
+def traverse(root: Path, *, follow: bool=True) -> Iterable[Path]:
+    if not root.is_dir():
+        yield root
+        return
+
+    from subprocess import Popen, PIPE
+
+    cmd = ['find', *find_args(root, follow=follow)]
+    # try to use fd.. it cooperates well with gitignore etc, also faster than find
+    for x in ('fd', 'fd-find', 'fdfind'): # has different names on different dists..
+        if shutil.which(x):
+            cmd = [x, *fdfind_args(root, follow=follow)]
+            break
+    else:
+        warnings.warn("'fdfind' is recommended for the best indexing performance. See https://github.com/sharkdp/fd#installation. Falling back to 'find'")
+
+    logger.debug('running: %s', cmd)
+    # TODO split by \0?
+    # TODO should it check return code? not sure
+    with Popen(cmd, stdout=PIPE) as p:
+        out = p.stdout
+        assert out is not None
+        for line in out:
+            fpath = Path(line.decode('utf8').strip())
+            yield fpath
+
+
+@lru_cache(1)
+def get_system_zone() -> str:
+    try:
+        import tzlocal # type: ignore
+        return tzlocal.get_localzone().zone
+    except Exception as e:
+        logger.exception(e)
+        logger.error("Couldn't determine system timezone. Falling back to UTC. Please report this as a bug!")
+        return 'UTC'
+
+
+@lru_cache(1)
+def get_system_tz():
+    zone = get_system_zone()
+    try:
+        return pytz.timezone(zone)
+    except Exception as e:
+        logger.exception(e)
+        logger.error(f"Unknown time zone %s. Falling back to UTC. Please report this as a bug!", zone)
+        return pytz.utc
+
+
+def file_mtime(path: PathIsh) -> datetime:
+    tz = get_system_tz()
+    return datetime.fromtimestamp(Path(path).stat().st_mtime, tz=tz)
+
+
+def now_tz() -> datetime:
+    return datetime.now(tz=get_system_tz())
+
+
+def user_config_file() -> Path:
+    return Path(appdirs().user_config_dir) / 'config.py'
+
+
+def default_config_path() -> Path:
+    cfg = Path('config.py')
+    if cfg.exists():
+        # todo ugh. might be nice to warn, but then it will always spam in cli...
+        # eh. not sure if it's a good idea, but whatever, it was the old behaviour
+        return cfg.absolute()
+    else:
+        return user_config_file()
+    # TODO need to test this..
