@@ -1,14 +1,15 @@
 from pathlib import Path
-from typing import List, Optional, Union, NamedTuple, Iterable
+import os
+from typing import List, Optional, Union, NamedTuple, Iterable, Callable
 import importlib
 import importlib.util
 import warnings
 
-import pytz
+from .common import PathIsh, get_tmpdir, appdirs, default_output_dir, default_cache_dir, user_config_file
+from .common import Res, Source, DbVisit
 
-from .common import PathIsh, get_tmpdir, appdirs, default_output_dir
-from .common import Res, Source
 
+HookT = Callable[[Res[DbVisit]], Iterable[Res[DbVisit]]]
 
 class Config(NamedTuple):
     # TODO remove default from sources once migrated
@@ -17,12 +18,15 @@ class Config(NamedTuple):
     # if not specified, uses user data dir
     OUTPUT_DIR: Optional[PathIsh] = None
 
-    CACHE_DIR: Optional[PathIsh] = None
+    CACHE_DIR: Optional[PathIsh] = ''
     FILTERS: List[str] = []
+
+    HOOK: Optional[HookT] = None
+
     #
     # NOTE: INDEXERS is deprecated, use SOURCES instead
     INDEXERS: List = []
-
+    #MIME_HANDLER: Optional[str] = None # TODO
     @property
     def sources(self) -> Iterable[Res[Source]]:
         idx = self.INDEXERS
@@ -52,15 +56,19 @@ class Config(NamedTuple):
                 yield Source(r)
 
     @property
-    def cache_dir(self) -> Path:
+    def cache_dir(self) -> Optional[Path]:
         cd = self.CACHE_DIR
-        # TODO maybe do not use cache if it's none?
-        assert cd is not None
-        res = Path(cd)
-        res.mkdir(exist_ok=True) # TODO not sure about parents=True
-        return res
+        cpath: Optional[Path]
+        if cd is None:
+            cpath = None # means 'disabled' in cachew
+        elif cd == '': # meh.. but need to make it None friendly..
+            cpath = default_cache_dir()
+        else:
+            cpath = Path(cd)
+        if cpath is not None:
+            cpath.mkdir(exist_ok=True, parents=True)
+        return cpath
 
-    # TODO make this optional, default to .cache or something?
     # TODO also tmp dir -- perhaps should be in cache or at least possible to specify in config? not sure if useful
     @property
     def output_dir(self) -> Path:
@@ -70,6 +78,13 @@ class Config(NamedTuple):
         else:
             return default_output_dir()
 
+    @property
+    def db(self) -> Path:
+        return self.output_dir / 'promnesia.sqlite'
+
+    @property
+    def hook(self) -> Optional[HookT]:
+        return self.HOOK
 
 instance: Optional[Config] = None
 
@@ -78,7 +93,7 @@ def has() -> bool:
     return instance is not None
 
 def get() -> Config:
-    assert instance is not None
+    assert instance is not None, "Expected config to be set, but it's not"
     return instance
 
 
@@ -107,3 +122,29 @@ def import_config(config_file: PathIsh) -> Config:
         if hasattr(mod, f):
             d[f] = getattr(mod, f)
     return Config(**d)
+
+
+# TODO: ugh. this causes warnings to be repeated multiple times... need to reuse the pool or something..
+def use_cores() -> Optional[int]:
+    '''
+    Somewhat experimental.
+    For now only used in sources.auto, perhaps later will be shared among the other indexers.
+    '''
+    # most likely needs to be some sort of pipeline thing?
+    cs = os.environ.get('PROMNESIA_CORES', None)
+    if cs is None:
+        return None
+    try:
+        return int(cs)
+    except ValueError: # any other value means 'use all
+        return 0
+
+
+def extra_fd_args() -> List[str]:
+    '''
+    Not sure where it belongs yet... so via env variable for now
+    Can be used to pass --ignore-file parameter
+    '''
+    v = os.environ.get('PROMNESIA_FD_EXTRA_ARGS', '')
+    extra = v.split() # eh, hopefully splitting that way is ok...
+    return extra
