@@ -10,6 +10,7 @@ from typing import List
 
 SYSTEM = platform.system()
 UNSUPPORTED_SYSTEM = RuntimeError(f'Platform {SYSTEM} is not supported yet!')
+NO_SYSTEMD = RuntimeError('systemd not detected, find your own way to start promnesia automatically')
 
 from .. import root
 from .. import server
@@ -94,7 +95,12 @@ def install_launchd(name: str, out: Path, launcher: str, largs: List[str]) -> No
 
 def install(args) -> None:
     name = args.name
+    # todo use appdirs for config dir detection
     if SYSTEM == 'Linux':
+        # Check for existence of systemd
+        # https://www.freedesktop.org/software/systemd/man/sd_booted.html
+        if not Path('/run/systemd/system/').exists():
+            raise NO_SYSTEMD
         suf = '.service'
         if Path(name).suffix != suf:
             name = name + suf
@@ -115,7 +121,13 @@ def install(args) -> None:
         exe = distutils.spawn.find_executable('promnesia'); assert exe is not None
         launcher = exe # older systemd wants absolute paths..
 
-    largs = ['serve', '--db', args.db, '--timezone', args.timezone, '--port', args.port]
+    db = args.db
+    largs = [
+        'serve',
+        *([] if db is None else ['--db', str(db)]),
+        '--timezone', args.timezone,
+        '--port', args.port,
+    ]
 
     out.parent.mkdir(parents=True, exist_ok=True) # sometimes systemd dir doesn't exist
     if SYSTEM == 'Linux':
@@ -132,7 +144,8 @@ def setup_parser(p: argparse.ArgumentParser) -> None:
     elif SYSTEM == 'Darwin':
         dflt = 'com.github.karlicoss.promnesia'
     else:
-        raise UNSUPPORTED_SYSTEM
+        # defensive here because setup_parser is called regardless whether the functionality is used
+        dflt = NotImplemented
 
     p.add_argument('--name', type=str, default=dflt, help='Systemd/launchd service name')
     p.add_argument('--unit-name', type=str, dest='name', help='DEPRECATED, same as --name')
