@@ -5,7 +5,7 @@ from datetime import datetime
 from tempfile import TemporaryDirectory
 from subprocess import check_call, check_output
 from time import sleep
-from typing import NamedTuple, Optional
+from typing import NamedTuple, Optional, Iterator
 
 import pytest # type: ignore
 
@@ -20,11 +20,16 @@ from selenium.webdriver.common.keys import Keys
 from selenium.webdriver.common.by import By
 
 
-from common import under_ci, skip_if_ci, uses_x
+from common import under_ci, skip_if_ci, uses_x, has_x
 from integration_test import index_hypothesis, index_local_chrome, index_urls
 from server_test import wserver
 from firefox_helper import open_extension_page
 
+from promnesia.logging import LazyLogger
+
+
+logger = LazyLogger('promnesia-tests')
+
 
 class Browser(NamedTuple):
     dist: str
@@ -164,10 +169,10 @@ def configure(
         show_dots: bool=True,
         highlights: Optional[bool]=None,
         blacklist=None,
-        notification: Optional[bool]=None,
+        notify_contexts: Optional[bool]=None,
         position: Optional[str]=None,
         verbose_errors: bool=True,
-):
+) -> None:
     def set_checkbox(cid: str, value: bool):
         cb = driver.find_element_by_id(cid)
         selected = cb.is_selected()
@@ -197,8 +202,8 @@ def configure(
     if highlights is not None:
         set_checkbox('highlight_id', highlights)
 
-    if notification is not None:
-        set_checkbox('contexts_popup_id', notification)
+    if notify_contexts is not None:
+        set_checkbox('contexts_popup_id', notify_contexts)
 
     if position is not None:
         set_position(driver, position)
@@ -299,24 +304,46 @@ class TestHelper(NamedTuple):
         check_call(['import', '-window', self.wid(), path])
 
 
-def confirm(what: str):
+def confirm(what: str) -> None:
     import click # type: ignore
-    click.confirm(what, abort=True)
+    click.confirm(click.style(what, blink=True, fg='yellow'), abort=True)
+
+
+class Manual:
+    def confirm(self, what: str) -> None:
+        raise NotImplementedError
+
+class Interactive(Manual):
+    def confirm(self, what: str) -> None:
+        confirm(what)
+
+class Headless(Manual):
+    def confirm(self, what: str) -> None:
+        logger.warning('"%s": headless mode, responding "yes"', what)
+
+
+'''
+Helper for tests that are not yet fully automated and require a human to check...
+- if running with the GUI, will be interactive
+- if running in headless mode, will automatically assume 'yes'.
+  of course it's not very robust, but at least we're testing some codepaths then
+'''
+manual = Interactive() if has_x() else Headless()
 
 
 @contextmanager
-def _test_helper(tmp_path, indexer, test_url: Optional[str], show_dots: bool=False, browser: Browser=FFH):
+def _test_helper(tmp_path, indexer, test_url: Optional[str], browser: Browser=FFH, **kwargs):
     tdir = Path(tmp_path)
 
     indexer(tdir)
     with wserver(db=tdir / 'promnesia.sqlite') as srv, get_webdriver(browser=browser) as driver:
         port = srv.port
-        configure_extension(driver, port=port, show_dots=show_dots)
+        configure(driver, port=port, **kwargs)
         sleep(0.5)
 
         if test_url is not None:
             driver.get(test_url)
-            sleep(3)
+            sleep(3) # todo use some condition...
         else:
             driver.get('about:blank')
 
@@ -329,11 +356,20 @@ class Command:
 # TODO assert this against manifest?
 
 
-def browsers(*br):
+def browsers(*br: Browser):
+    if len(br) == 0:
+        br = (FF, FFH, CH)
+    if not has_x():
+        br = tuple(b for b in br if b.headless)
+
     from functools import wraps
     def dec(f):
+        if len(br) == 0:
+            dd = pytest.mark.skip('Filtered out all browsers (because of no GUI/non-interactive mode)')
+        else:
+            dd = pytest.mark.parametrize('browser', br, ids=lambda b: b.dist.replace('-', '_') + ('_headless' if b.headless else ''))
         @pytest.mark.with_browser
-        @pytest.mark.parametrize('browser', br, ids=lambda b: b.dist.replace('-', '_') + ('_headless' if b.headless else ''))
+        @dd
         @wraps(f)
         def ff(*args, **kwargs):
             return f(*args, **kwargs)
@@ -344,19 +380,15 @@ def browsers(*br):
 PYTHON_DOC_URL = 'file:///usr/share/doc/python3/html/index.html'
 
 
-@browsers(FFH, CH)
+@browsers()
 def test_installs(tmp_path, browser):
-    browser.skip_ci_x()
-
     with get_webdriver(browser=browser):
         # just shouldn't crash
         pass
 
 
-@browsers(FFH, CH)
+@browsers()
 def test_settings(tmp_path, browser):
-    browser.skip_ci_x()
-
     # TODO fixture for driver?
     with get_webdriver(browser=browser) as driver:
         configure_extension(driver, port='12345', show_dots=False)
@@ -366,10 +398,8 @@ def test_settings(tmp_path, browser):
         assert hh.get_attribute('value') == 'http://localhost:12345'
 
 
-@browsers(FFH, FF, CH)
+@browsers()
 def test_backend_status(tmp_path, browser):
-    browser.skip_ci_x()
-
     with get_webdriver(browser=browser) as driver:
         open_extension_page(driver, page='options_page.html')
         sleep(1) # ugh. for some reason pause here seems necessary..
@@ -380,14 +410,8 @@ def test_backend_status(tmp_path, browser):
         alert = driver.switch_to.alert
         assert 'ERROR' in alert.text
 
-        # TODO ugh, not sure why it just stucks on headless...
-        # perhaps use policy instead? https://stackoverflow.com/a/43946973/706389
-        if not browser.headless:
-            driver.switch_to.alert.accept()
-            sleep(0.5)
-            # ugh. extra alert...
-            driver.switch_to.alert.accept()
-
+        sleep(0.5) #  don't remember if needed?
+        driver.switch_to.alert.accept()
         # TODO implement positive check??
 
 def set_position(driver, settings: str):
@@ -408,11 +432,9 @@ def set_position(driver, settings: str):
         area.send_keys(settings)
 
 
-
+# TODO ffh?
 @browsers(FF, CH)
 def test_sidebar_bottom(browser):
-    browser.skip_ci_x()
-
     with get_webdriver(browser=browser) as driver:
         open_extension_page(driver, page='options_page.html')
         sleep(1) # ugh. for some reason pause here seems necessary..
@@ -436,27 +458,23 @@ def test_sidebar_bottom(browser):
         confirm("You should see sidebar below")
 
 
-@uses_x
 @browsers(FF, CH)
-def test_blacklist_custom(tmp_path, browser):
-    # TODO make confirm a fixture? so we can automatically skip them on ci
+def test_blacklist_custom(tmp_path, browser) -> None:
     with get_webdriver(browser=browser) as driver:
         configure_extension(driver, port='12345', blacklist=('stackoverflow.com',))
         driver.get('http://stackoverflow.com')
         trigger_command(driver, Command.ACTIVATE)
-        confirm('page should be blacklisted (black icon), your should see an error notification')
+        manual.confirm('page should be blacklisted (black icon), your should see an error notification')
 
 
-@uses_x
-@browsers(FF, CH)
+@browsers()
 def test_blacklist_builtin(tmp_path, browser):
     with get_webdriver(browser=browser) as driver:
         configure_extension(driver, port='12345')
         driver.get('https://www.hsbc.co.uk/mortgages/')
-        confirm('page should be blacklisted (black icon)')
+        manual.confirm('page should be blacklisted (black icon)')
 
 
-@uses_x
 @browsers(FF, CH)
 def test_add_to_blacklist(tmp_path, browser):
     with get_webdriver(browser=browser) as driver:
@@ -483,7 +501,6 @@ def test_add_to_blacklist(tmp_path, browser):
         confirm('page should be blacklisted (black icon)')
 
 
-@uses_x
 @browsers(FF, CH)
 def test_visits(tmp_path, browser):
     test_url = "http://www.e-flux.com/journal/53/59883/the-black-stack/"
@@ -493,16 +510,16 @@ def test_visits(tmp_path, browser):
         confirm('you should see hypothesis contexts')
 
 
-@uses_x
-@browsers(FF, CH)
-def test_search_around(tmp_path, browser):
+@browsers()
+def test_search_around(tmp_path, browser) -> None:
     # TODO it actually lacks a proper end-to-end test withing browser... although I do have something in automatic demos?
     test_url = "about:blank"
     with _test_helper(tmp_path, index_hypothesis, test_url, browser=browser) as h:
         # TODO hmm. dunno if we want to highlight only result with the same timestamp, or the results that are 'near'??
         ts = int(datetime.strptime("2017-05-22T10:59:12.082375+00:00", '%Y-%m-%dT%H:%M:%S.%f%z').timestamp())
-        h.open_page(f'search.html?utc_timestamp={ts}')
-        confirm('you should see search results, "anthrocidal" should be highlighted red')
+        h.open_page(f'search.html?utc_timestamp_s={ts}')
+        manual.confirm('you should see search results, "anthrocidal" should be highlighted red')
+        # FIXME test clicking search around in actual search page.. it didn't work, seemingly because of initBackground() handling??
 
 
 # TODO skip if not my hostname
@@ -516,7 +533,6 @@ def test_chrome_visits(tmp_path, browser):
         confirm("You shoud see chrome visits now; with time spent")
 
 
-@uses_x
 @browsers(FF, CH)
 def test_show_dots(tmp_path, browser):
     visited = {
@@ -530,35 +546,37 @@ def test_show_dots(tmp_path, browser):
         confirm("You should see dots near special linear group, Unitary group, Transpose")
 
 
-@uses_x
 @browsers(FF, CH)
 def test_search(tmp_path, browser):
     test_url = "https://en.wikipedia.org/wiki/Symplectic_vector_space"
-    with _test_helper(tmp_path, index_local_chrome, test_url, browser=browser) as helper:
+    with _test_helper(tmp_path, index_hypothesis, test_url, browser=browser) as helper:
         trigger_command(helper.driver, Command.SEARCH)
         # TODO actually search something?
         # TODO use current domain as deafult? or 'parent' url?
         confirm("You shoud see search prompt now, with focus on search field")
 
 
-@uses_x
-@browsers(FF, CH)
-def test_new_background_tab(tmp_path, browser):
+@browsers()
+def test_new_background_tab(tmp_path, browser) -> None:
     start_url = "http://www.e-flux.com/journal/53/59883/the-black-stack/"
     # bg_url_text = "El Proceso (The Process)"
     # TODO generate some fake data instead?
-    with _test_helper(tmp_path, index_hypothesis, start_url, browser=browser) as helper:
-        confirm('you should see notification about contexts')
+    with _test_helper(
+            tmp_path, index_hypothesis, start_url, browser=browser,
+            notify_contexts=True,
+    ) as helper:
+        manual.confirm('you should see notification about contexts')
         helper.driver.find_element(By.XPATH, '//div[@class="logo"]/a').send_keys(Keys.CONTROL + Keys.ENTER)
-        confirm('you should not see any new notifications')
+        manual.confirm('you should not see any new notifications')
         # TODO switch to new tab?
         # TODO https://www.e-flux.com/journal/53/
 
 
-@uses_x
 # TODO shit disappears on chrome and present on firefox
 @browsers(FF, CH)
-def test_local_page(tmp_path, browser):
+# FIXME fails on headless firefox because in docker python3-doc isn't installing this file??
+# really weird, mismatches dpkg -L
+def test_local_page(tmp_path, browser) -> None:
     tutorial = 'file:///usr/share/doc/python3/html/tutorial/index.html'
     urls = {
          tutorial                                                : 'TODO read this',
@@ -572,49 +590,48 @@ def test_local_page(tmp_path, browser):
         helper.driver.back()
         # TODO it's always guaranteed to work? https://stackoverflow.com/questions/27626783/python-selenium-browser-driver-back
         confirm('grey icon, should be no sidebar')
+        # fIXME fuck. failing here to load anchorme as well as webext-options-sync? ugh.
         helper.driver.forward()
         confirm('green icon, sidebar visible')
 
 
-@uses_x
-@browsers(FF, CH)
-def test_unreachable(tmp_path, browser):
+@browsers()
+def test_unreachable(tmp_path, browser) -> None:
     url = 'https://somenonexist1ngurl.com'
     urls = {
         url: 'some context',
     }
-    with _test_helper(tmp_path, index_urls(urls), 'about:blank', browser=browser) as helper:
+    with _test_helper(
+            tmp_path, index_urls(urls), 'about:blank', browser=browser,
+            notify_contexts=True,
+            verbose_errors=False,
+    ) as helper:
         try:
             helper.driver.get(url)
         except:
             # results in exception because it's unreachable
             pass
-        confirm('green icon, no errors, desktop notification with contexts')
+        # TODO maybe in this case it could instead open the sidebar in a separate tab?
+        manual.confirm('green icon, no errors, desktop notification with contexts')
 
 
-@uses_x
-@browsers(FF, CH)
-def test_stress(tmp_path, browser):
-    page = tmp_path / 'dummy.html'
-    page.write_text('''
-<html>
-<head>HI</head>
-<body>test</body>
-</html>
-    ''')
-    url = f'file://{page}'
-
+@browsers()
+def test_stress(tmp_path, browser) -> None:
+    url = 'https://www.reddit.com/'
     urls = [
-        (url, f'ctx {i}' if i < 20 else None) for i in range(10000)
+        (f'{url}/subpath/{i}.html', f'context {i}' if i > 10000 else None) for i in range(50000)
     ]
     with _test_helper(tmp_path, index_urls(urls), url, browser=browser) as helper:
-        # TODO so, should the response be immediate?
-        # TODO shouldn't do any work if we don't open the sidebar
-        confirm('is performance reasonable?')
+        if has_x():
+            helper.activate()
+
+        manual.confirm('''
+Is performance reasonable?
+The sidebar should show up, and update gradually.
+You should be able to scroll the page, trigger tooltips, etc., without any lags.
+'''.strip())
 
     
-
-@uses_x
 @browsers(FF, CH)
 def test_fuzz(tmp_path, browser):
     # TODO ugh. this still results in 'tab permissions' pages, but perhaps because of closed tabs?
@@ -640,6 +657,7 @@ def test_fuzz(tmp_path, browser):
                 send_key('Ctrl+Shift+t')
                 sleep(0.1)
         trigger_callback(driver, cb)
+        # FIXME shows quite a bunch of notifications here...
         confirm("shouldn't result in 'unexpected error occured'; show only show single notification per page")
 
 
@@ -650,7 +668,6 @@ def trigger_sidebar_search(driver):
     search_button.click()
 
 
-@uses_x
 @browsers(FF, CH)
 def test_duplicate_background_pages(tmp_path, browser):
     url = PYTHON_DOC_URL
