@@ -1,4 +1,5 @@
 #!/usr/bin/env python3
+from contextlib import contextmanager
 from datetime import datetime, date
 from os.path import join, getsize
 from pathlib import Path
@@ -8,16 +9,16 @@ from shutil import copytree
 import os
 from os import mkdir
 from os.path import lexists
-from typing import Union
+from typing import Union, List
 
 import pytest # type: ignore
 from pytest import mark # type: ignore
 
 
 from common import skip_if_ci, tdata
+from config_tests import with_config
 
-from promnesia.common import History, Visit
-from promnesia.common import Indexer
+from promnesia.common import Visit, Indexer, Loc, Res, DbVisit
 
 # TODO need to expire dbcache in tests..
 
@@ -29,9 +30,20 @@ def W(*args, **kwargs):
         kwargs['src'] = 'whatever'
     return Indexer(*args, **kwargs)
 
-def history(*args, **kwargs):
-    from promnesia.common import previsits_to_history
-    return previsits_to_history(*args, **kwargs, src='whatever')[0] # TODO meh
+
+def as_visits(*args, **kwargs) -> List[Res[DbVisit]]:
+    from promnesia.extract import extract_visits
+    kwargs['src'] = 'whatever'
+    return list(extract_visits(*args, **kwargs))
+
+
+def as_ok_visits(*args, **kwargs) -> List[DbVisit]:
+    r: List[DbVisit] = []
+    for v in as_visits(*args, **kwargs):
+        if isinstance(v, Exception):
+            raise v
+        r.append(v)
+    return r
 
 
 from contextlib import contextmanager
@@ -58,7 +70,6 @@ def import_file(p: Union[str, Path], name=None):
 
 # TODO eh. need to separate stuff for history backups out...
 backup_db = import_file('scripts/browser_history.py')
-populate_db = import_file('scripts/populate-browser-history.py')
 
 def assert_got_tzinfo(visits):
     for v in visits:
@@ -91,17 +102,16 @@ def test_takeout_directory(adhoc_config, tmp_path):
     from my.cfg import config
     class user_config:
         takeout_path = tdata('takeout')
-    config.google = user_config
+    config.google = user_config # type: ignore
     import promnesia.sources.takeout as tex
 
-    visits = history(W(tex.index))
+    visits = as_visits(W(tex.index))
     assert len(visits) > 0 # kinda arbitrary?
 
     assert_got_tzinfo(visits)
 
 
-
-def test_with_error():
+def test_with_error() -> None:
     class ExtractionError(Exception):
         pass
     def err_ex():
@@ -112,20 +122,22 @@ def test_with_error():
                 yield Visit(
                     url=f'http://test{i}',
                     dt=datetime.utcfromtimestamp(0),
-                    locator=None,
+                    locator=Loc.make('whatever'),
                 )
-    hist = history(lambda: err_ex())
-    assert len(hist) == 2
+    [v1, e, v2] = as_visits(lambda: err_ex())
+    assert isinstance(v1, DbVisit)
+    assert isinstance(e, Exception)
+    assert isinstance(v2, DbVisit)
 
 
-def test_takeout_new_zip(adhoc_config):
+def test_takeout_new_zip(adhoc_config) -> None:
     from my.cfg import config
     class user_config:
         takeout_path = tdata('takeout-20150518T000000Z.zip')
-    config.google = user_config
+    config.google = user_config # type: ignore
 
     import promnesia.sources.takeout as tex
-    visits = history(tex.index)
+    visits = as_ok_visits(tex.index)
     assert len(visits) == 3
     [vis] = [v for v in visits if v.norm_url == 'takeout.google.com/settings/takeout']
 
@@ -143,24 +155,8 @@ def test_takeout_new_zip(adhoc_config):
     assert_got_tzinfo(visits)
 
 
-# TODO run condition?? and flag to force all
 @skip_if_ci("TODO try triggering firefox on CI? not sure if that's possible...")
-def test_chrome(tmp_path):
-    from promnesia.sources.browser import chrome
-    tdir = Path(tmp_path)
-
-    path = tdir / 'history'
-    populate_db.merge_from('chrome', from_=None, to=path)
-    # TODO hmm, it actually should be from merged db....
-
-    hist = history(W(chrome, path))
-    assert len(hist) > 10 # kinda random sanity check
-
-    assert_got_tzinfo(hist)
-
-
-@skip_if_ci("TODO try triggering firefox on CI? not sure if that's possible...")
-def test_firefox(tmp_path):
+def test_backup_firefox(tmp_path):
     tdir = Path(tmp_path)
     path = backup_db.backup_history('firefox', to=tdir, profile='*release*')
     # shouldn't fail at least
@@ -173,11 +169,11 @@ def test_firefox(tmp_path):
         # assert_got_tzinfo(hist)
 
 
-def test_plaintext_path_extractor():
+def test_plaintext_path_extractor() -> None:
     import promnesia.sources.shellcmd as custom_gen
     from promnesia.sources.plaintext import extract_from_path
 
-    visits = history(W(custom_gen.index,
+    visits = as_ok_visits(W(custom_gen.index,
         extract_from_path(tdata('custom')),
     ))
     assert {
@@ -185,7 +181,7 @@ def test_plaintext_path_extractor():
     } == {
         'http://google.com',
         'http://google.com/',
-        'http://some-weird-domain/whatever',
+        'http://some-weird-domain.xyz/whatever',
         'https://google.com',
         'http://what.about.this.link',
     }
@@ -195,11 +191,11 @@ def test_plaintext_path_extractor():
     assert wa.locator.href == f'editor://{f2}:3' # occurs line 3
 
 # TODO perhaps it belongs to canonify?
-def test_normalise():
+def test_normalise() -> None:
     import promnesia.sources.shellcmd as custom_gen
     from promnesia.sources.plaintext import extract_from_path
 
-    visits = history(W(custom_gen.index,
+    visits = as_ok_visits(W(custom_gen.index,
         extract_from_path(tdata('normalise')),
     ))
     assert len(visits) == 7
@@ -214,43 +210,106 @@ def test_normalise():
     }
 
 
-def test_normalise_weird():
+def test_normalise_weird() -> None:
     import promnesia.sources.shellcmd as custom_gen
     from promnesia.sources.plaintext import extract_from_path
 
-    visits = history(W(
+    visits = as_ok_visits(W(
         custom_gen.index,
         extract_from_path(tdata('weird.txt')),
     ))
-    norms = {v.norm_url for v in visits}
+    [v1, v2] = visits
 
     # TODO assert there are no spaces in the database?
-    assert "urbandictionary.com/define.php?term=Belgian%20Whistle" in norms
-    assert "en.wikipedia.org/wiki/Dinic%27s_algorithm" in norms
+    assert "urbandictionary.com/define.php?term=Belgian%20Whistle" == v1.norm_url
+
+    assert "en.wikipedia.org/wiki/Dinic%27s_algorithm"             == v2.norm_url
+    assert v2.locator.title.endswith('weird.txt:2')
+    assert v2.context == 'right, so https://en.wikipedia.org/wiki/Dinic%27s_algorithm can be used for max flow'
 
 
-@skip("use a different way to specify filter other than class variable..")
-def test_filter():
+def test_filter() -> None:
     import promnesia.sources.shellcmd as custom_gen
     from promnesia.sources.plaintext import extract_from_path
 
-    History.add_filter(r'some-weird-domain')
-    hist = custom_gen.get_custom_history(
-        extract_from_path(tdata('custom')),
-    )
-    assert len(hist) == 4 # chrome-error got filtered out
+    # ugh... such a mess
+    @contextmanager
+    def reset_filters():
+        try:
+            E.filters.cache_clear()
+            yield
+        finally:
+            E.filters.cache_clear()
 
-def test_custom():
+    import promnesia.extract as E
+    with reset_filters(), with_config('''
+FILTERS = [
+    "some-weird-domain.xyz"
+]
+'''):
+        visits = as_visits(W(
+            custom_gen.index,
+            extract_from_path(tdata('custom')),
+        ))
+        assert len(visits) == 4
+
+
+def test_custom() -> None:
     import promnesia.sources.shellcmd as custom_gen
 
-    hist = history(W(
+    visits = as_visits(W(
         custom_gen.index,
         """grep -Eo -r --no-filename '(http|https)://\S+' """ + tdata('custom'),
     ))
     # TODO I guess filtering of equivalent urls should rather be tested on something having context (e.g. org mode)
-    assert len(hist) == 5
+    assert len(visits) == 5
 
 
+def test_hook() -> None:
+    import promnesia.sources.shellcmd as custom_gen
+    from promnesia.__main__ import iter_all_visits
+    with with_config('''
+from promnesia import Source
+from promnesia.sources import demo
+
+SOURCES = [
+    Source(demo.index, count=7, name='somename'),
+]
+
+from typing import Iterable
+from promnesia import DbVisit, Loc, Res
+
+def HOOK(visit: Res[DbVisit]) -> Iterable[Res[DbVisit]]:
+    # NOTE: might be a good idea to check that the visit is an exception first and yield it intact?
+    nurl = visit.norm_url
+    if 'page1' in nurl:
+        yield visit._replace(norm_url='patched.com')
+    elif 'page2' in nurl:
+        None.boom # deliberately crash
+    elif 'page3' in nurl:
+        # just don't yield anything! it will be omitted
+        pass
+    elif 'page4' in nurl:
+         # can emit multiple!
+        yield visit
+        yield visit
+    elif 'page6' in nurl:
+        # patch locator
+        yield visit._replace(locator=Loc.make(title='some custom timte', href='/can/replace/original/path'))
+    else:
+        yield visit
+'''):
+        # TODO hmm might be nice to allow in-pace modifications...
+        [p0, p1, e2, p41, p42, p5, p6] = list(iter_all_visits())
+        assert isinstance(p0, DbVisit)
+        assert p0.norm_url == 'demo.com/page0.html'
+        assert isinstance(p1, DbVisit)
+        assert p1.norm_url == 'patched.com'
+        assert isinstance(e2, Exception)
+        assert p41 == p42
+        assert isinstance(p6, DbVisit)
+        assert p6.locator is not None
+
 
 TESTDATA_CHROME_HISTORY = "/L/data/promnesia/testdata/chrome-history"
 
@@ -311,9 +370,9 @@ def _test_merge_all_from(tdir):
     assert not lexists(first)
     assert not lexists(second)
 
-    import promnesia.sources.chrome as chrome_ex
+    import promnesia.sources.chrome as chrome_ex # type: ignore
 
-    hist = history(W(chrome_ex.extract, mfile))
+    hist = history(W(chrome_ex.extract, mfile)) # type: ignore
     assert len(hist) > 0
 
     older = hist['github.com/orgzly/orgzly-android/issues']
