@@ -1,8 +1,8 @@
+import os
 from pathlib import Path
 from subprocess import check_call, run
-from typing import Set, Dict, Optional, Union, Sequence, Tuple
+from typing import Set, Dict, Optional, Union, Sequence, Tuple, Mapping, List
 
-from indexer_test import populate_db
 from common import tdir, under_ci, DATA, GIT_ROOT
 
 import pytest
@@ -11,22 +11,25 @@ import platform
 system = platform.system()
 
 
-def index(cfg: Path):
+def run_index(cfg: Path, *, update=False) -> None:
     from promnesia.__main__ import do_index
-    do_index(cfg)
+    if update:
+        ev = 'PROMNESIA_INDEX_POLICY'
+        os.environ[ev] = 'update'
+        try:
+            do_index(cfg)
+        finally:
+            del os.environ[ev]
+    else:
+        do_index(cfg)
 
 
-def test_example_config(tdir):
-    example = GIT_ROOT / 'config.py.example'
-    ex = example.read_text()
-    if under_ci():
-        # TODO ugh fucking hell I couldn't find a single path that has HTMLs both on macos and ubuntu
-        # and using /usr/share/docs locally might index a bit too much
-        if system == 'Darwin':
-            repl = '/usr/share/doc/cups/'
-        else:
-            repl = '/usr/share/doc/python3/'
-        ex = ex.replace('/usr/share/doc/python3/html/faq', repl)
+index = run_index # legacy name
+
+
+def test_example_config(tdir) -> None:
+    from promnesia.__main__ import read_example_config
+    ex = read_example_config()
     cfg = tdir / 'test_config.py'
     cfg.write_text(ex)
     index(cfg)
@@ -54,7 +57,7 @@ def _get_stuff(outdir: Path):
 
 # TODO a bit shit... why did I make it dict at first??
 Urls = Union[
-              Dict[str, Optional[str]],
+           Mapping[str, Optional[str]],
     Sequence[Tuple[str, Optional[str]]],
 ]
 
@@ -85,17 +88,9 @@ SOURCES = [indexer]
     return idx
 
 
-def index_hypothesis(tdir: Path):
-    # TODO meh..
-    # TODO use submodule?
-    hypexport_path = Path(tdir) / 'hypexport'
-    check_call([
-        'git',
-        'clone',
-        # TODO do I need --recursive here?? I guess it's only for fetching new data?
-        'https://github.com/karlicoss/hypexport',
-        str(hypexport_path),
-    ])
+def index_hypothesis(tdir: Path) -> None:
+    hypexport_path  = DATA / 'hypexport'
+    hypothesis_data = hypexport_path / 'testdata'
 
     cfg = tdir / 'test_config.py'
     # TODO ok, need to simplify this...
@@ -107,10 +102,13 @@ from promnesia.common import Source
 def hyp_extractor():
     import my.config
     class user_config:
-        export_path = '{str(DATA)}/hypothesis/netrights-dashboards-mockup/data/*.json'
-        hypexport   = '{hypexport_path}'
+        export_path = '{str(hypothesis_data)}/netrights-dashboard-mockup/data/*.json'
     my.config.hypothesis = user_config
 
+    # todo ideally would use virtualenv?
+    import sys
+    sys.path.insert(0, "{str(hypexport_path / 'src')}")
+
     import promnesia.sources.hypothesis as hypothesis
     yield from hypothesis.index()
 
@@ -135,6 +133,7 @@ def index_local_chrome(tdir: Path):
 OUTPUT_DIR = '{tdir}'
 
 from promnesia.common import Indexer as I
+# TODO FIXME -- fix back
 from promnesia.sources.browser import chrome
 
 chrome_extractor = I(chrome, '{merged}', src='chrome')
@@ -144,26 +143,28 @@ SOURCES = [chrome_extractor]
     index(cfg)
 
 
-def test_hypothesis(tdir):
-    index_hypothesis(tdir)
-
+def query_db_visits(tdir: Path) -> List[DbVisit]:
     # TODO copy pasting from server; need to unify
     engine, binder, table = _get_stuff(tdir)
     query = table.select()
     with engine.connect() as conn:
-        visits = [binder.from_row(row) for row in conn.execute(query)]
+        return [binder.from_row(row) for row in conn.execute(query)]
 
+
+def test_hypothesis(tdir: Path) -> None:
+    index_hypothesis(tdir)
+    visits = query_db_visits(tdir)
     assert len(visits) > 100
 
-    [vis] = [x for x in visits if 'fundamental fact of evolution' in x.context]
+    [vis] = [x for x in visits if 'fundamental fact of evolution' in (x.context or '')]
 
     assert vis.norm_url == 'wired.com/2017/04/the-myth-of-a-superhuman-ai'
     assert vis.orig_url == 'https://www.wired.com/2017/04/the-myth-of-a-superhuman-ai/'
     assert vis.locator.href == 'https://hyp.is/_Z9ccmVZEeexBOO7mToqdg/www.wired.com/2017/04/the-myth-of-a-superhuman-ai/'
-    assert 'misconception about evolution is fueling misconception about AI' in vis.context # contains notes as well
+    assert 'misconception about evolution is fueling misconception about AI' in (vis.context or '') # contains notes as well
 
 
-def test_comparison(tdir: Path):
+def test_comparison(tdir: Path) -> None:
     from promnesia.compare import compare_files
 
     idx = index_urls({
@@ -175,7 +176,7 @@ def test_comparison(tdir: Path):
     db     = tdir / 'promnesia.sqlite'
     old_db = tdir / 'promnesia-old.sqlite'
     import shutil
-    shutil.move(db, old_db)
+    shutil.move(str(db), str(old_db))
 
     idx2 = index_urls({
         'https://example.com': None,
@@ -191,7 +192,7 @@ def test_comparison(tdir: Path):
     assert len(list(compare_files(db, old_db))) == 1
 
 
-def test_index_many(tdir):
+def test_index_many(tdir: Path) -> None:
     # NOTE [20200521] experimenting with promnesia.dump._CHUNK_BY
     # inserting 100K visits
     # value=1000: 9 seconds
@@ -216,11 +217,32 @@ SOURCES = [Source(index)]
 OUTPUT_DIR = '{tdir}'
     """)
     index(cfg)
-    #
-    # TODO copy pasting from server; need to unify
-    engine, binder, table = _get_stuff(tdir)
-    query = table.select()
-    with engine.connect() as conn:
-        visits = [binder.from_row(row) for row in conn.execute(query)]
+    visits = query_db_visits(tdir)
 
     assert len(visits) == 100000
+
+
+def test_indexing_error(tdir: Path) -> None:
+    cfg = tdir / 'test_config.py'
+    cfg.write_text(f'''
+def bad_index():
+    import bad_import
+    yield from [] # dummy
+from promnesia import Source
+from promnesia.sources import demo
+
+SOURCES = [
+    bad_index,
+    Source(demo, count=3),
+]
+OUTPUT_DIR = '{tdir}'
+''')
+    with pytest.raises(SystemExit):
+        index(cfg)
+        # should exit(1)
+
+    # yet save the database
+    visits = query_db_visits(tdir)
+
+    [e, _, _, _] = visits
+    assert e.src == 'error'
